//      

const t = require('babel-types');
// import type * from 'babel-types';

function newVisitor() {
  return {
    leave: {
      Root(node) {
        const statements = [];
        if (node.operation !== 'query') {
          statements.push(
            generateInputVariablesType(
              node,
              customScalars,
              inputFieldWhiteList,
            ),
          );
        }
        statements.push(
          exportType(
            `${node.name}Response`,
            selectionsToBabel(node.selections, customScalars),
          ),
        );
        return t.program(statements);
      },

      Fragment(node) {
        let selections = flattenArray(node.selections);
        const numConecreteSelections = selections.filter(s => s.concreteType)
          .length;
        selections = selections.map(selection => {
          if (
            numConecreteSelections <= 1 &&
            isTypenameSelection(selection) &&
            !isAbstractType(node.type)
          ) {
            return [
              {
                ...selection,
                concreteType: node.type.toString(),
              },
            ];
          }
          return [selection];
        });
        const baseType = selectionsToBabel(selections, customScalars);
        const type = isPlural(node) ? readOnlyArrayOfType(baseType) : baseType;

        return t.program([exportType(node.name, type)]);
      },

      InlineFragment(node) {
        const typeCondition = node.typeCondition;
        return flattenArray(node.selections).map(typeSelection => {
          return isAbstractType(typeCondition)
            ? {
                ...typeSelection,
                conditional: true,
              }
            : {
                ...typeSelection,
                concreteType: typeCondition.toString(),
              };
        });
      },
      Condition(node) {
        return flattenArray(node.selections).map(selection => {
          return {
            ...selection,
            conditional: true,
          };
        });
      },
      ScalarField(node) {
        return [
          {
            key: node.alias || node.name,
            schemaName: node.name,
            value: transformScalarType(node.type, customScalars),
          },
        ];
      },
      LinkedField(node) {
        return [
          {
            key: node.alias || node.name,
            schemaName: node.name,
            nodeType: node.type,
            nodeSelections: selectionsToMap(flattenArray(node.selections)),
          },
        ];
      },
      FragmentSpread(node) {
        return [];
      },
    },
  };
}

function typeAlias(named                          )                {

  return [];

};
  
function astToScala(ast           )         {
  let strs = [];

  ast.body.map((statement) => {

    if (statement.type == "ExportNamedDeclaration") {
      strs = strs.concat(typeAlias(statement.declaration))
    }
  }
  )
  return strs.join("\n");
};

module.exports = {
  newVisitor
};